﻿using DingTalk.Api;
using DingTalk.Api.Request;
using DingTalk.Api.Response;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace BI.Controllers
{
    public class DingDingApis
    {
        public static string https_post_request(string host, string postdate, string method, string contenttype)
        {
            //格林威治时间
            DateTime dt = DateTime.UtcNow;
            //根据网站的编码自定义
            Encoding encoding = Encoding.UTF8;
            //初始化一个HttpWebRequest请求实例
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(host);
            //获取或设置cookie
            request.CookieContainer = new CookieContainer();
            //Http 请求方法 这里默认为POST
            request.Method = method;
            request.ServicePoint.Expect100Continue = false;
            //Http Post提交的数据的字节数
            request.ContentLength = Encoding.UTF8.GetByteCount(postdate);
            //Http 请求内容类型（一般为：application/xml）
            request.ContentType = contenttype;
            //Http 请求时间
            request.Date = dt;
            //POST数据字符串转成字节
            byte[] postData = encoding.GetBytes(postdate);
            //post的内容长度
            request.ContentLength = postData.Length;
            //设置响应超时时间
            request.Timeout = 60000;
            string retString = "";
            StreamReader streamReader = StreamReader.Null;
            Stream responseStream = Stream.Null;
            HttpWebResponse response = null;

            //向内存中写入POSTDATA数据流
            Stream requestStream = request.GetRequestStream();
            requestStream.Write(postData, 0, postData.Length);
            try
            {
                //获取对应HTTP请求的响应
                response = (HttpWebResponse)request.GetResponse();
            }
            catch (WebException ex)
            {
                //获取抛出异常的请求响应
                response = (HttpWebResponse)ex.Response;

            }
            responseStream = response.GetResponseStream();
            //如果http头中接受gzip的话，这里就要判断是否为有压缩，有的话，直接解压缩即可
            if (response.Headers["Content-Encoding"] != null && response.Headers["Content-Encoding"].ToLower().Contains("gzip"))
            {
                responseStream = new GZipStream(responseStream, CompressionMode.Decompress);
            }

            streamReader = new StreamReader(responseStream, encoding);
            retString = streamReader.ReadToEnd();
            streamReader.Close();
            streamReader.Dispose();
            responseStream.Close();
            responseStream.Dispose();
            response.Close();
            return retString;
        }

        /// <summary>
        /// Signature 签名算法
        /// </summary>
        /// <param name="key"></param>
        /// <param name="text"></param>
        /// <returns></returns>
        public static string HmacSign(string message, string secret)
        {
            secret = secret ?? "";
            var encoding = new System.Text.ASCIIEncoding();
            byte[] keyByte = System.Text.Encoding.UTF8.GetBytes(secret);
            byte[] messageBytes = System.Text.Encoding.UTF8.GetBytes(message);
            using (var hmacsha256 = new HMACSHA256(keyByte))
            {
                byte[] hashmessage = hmacsha256.ComputeHash(messageBytes);
                return System.Web.HttpUtility.UrlEncode(Convert.ToBase64String(hashmessage));
            }
        }
        /// <summary>
        /// 获取时间戳（毫秒）
        /// </summary>
        /// <returns></returns>
        public static string GetTimeStamp()
        {
            TimeSpan ts = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0);
            return Convert.ToInt64(ts.TotalMilliseconds).ToString();
        }
    }
}
